//#define __DEBUG__
#include <iomanip>
#ifndef __CINT__
#include <sstream>
#include "TROOT.h"
#include "TSystem.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH2.h"
#include "TTree.h"

#include "EXTPCKalDetector.h"
#include "kaltest/TKalDetCradle.h"

#include "JREvent.h"
#include "JCluster.h"
#include "JHitMaker.h"
#include "JHitCorrector.h"
#include "JTrackMaker.h"
#include "JSimpleTrackFitter.h"
#include "JTripletTrackFinder.h"
#include "JTrack.h"
#endif
//-----------------------------------//
//  Pulse shape (height/width) cuts  //
//-----------------------------------//
///////////////////////////////////////
//const double kCosThetaCut = 0.98;
//const int    kMaxMissing  =  15;
//const int    kMinNhits    =  10;
//const int    kZdiffCut    = 2.0;
const double kCosThetaCut = 0.98;
//const double kCosThetaCut = 0.97;
const int    kMaxMissing  =  15;
//const int    kMinNhits    =  20;
//const int    kMinNhits    =  10;
const int    kMinNhits    =  22;
const int    kZdiffCut    = 5.0;
//const int    kZdiffCut    = 2.0;
//const int    kZdiffCut    = 3.0;
///////////////////////////////////////

void reco(Int_t run = 17319, const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17324, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17327, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17331, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17340, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17344, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17352, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17356, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17359, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17364, Double_t p = 5., const Char_t *delxfile = "Results/deltax0.dat")
//void reco(Int_t run = 17367, Double_t p = 5.0, const Char_t *delxfile = "Results/deltax0.dat")
{
#ifdef __CINT__
  gSystem->Load("libTree");
  gSystem->Load("libGraf3d");
  gSystem->Load("libPhysics");
  gSystem->Load("libKalTest");
  gSystem->Load("libyoka");
#endif

#if 0
  gInterpreter->AddIncludePath("../../../KalTest/v02-03/include");
  gInterpreter->AddIncludePath("../../include");
#endif

  // ---------------
  // Open input file
  // ---------------
  stringstream finstr;
  finstr << "../unpacker/root/readout-" << run << "_0.root" << ends;
  cerr << "Infile: " << finstr.str().data() << endl;
  TFile *fp = new TFile(finstr.str().data());
  TFile &f  = *fp;
  TTree *T  = static_cast<TTree *>(f.Get("Event"));
  JREvent *event = 0;
  T->SetBranchAddress("Event", &event);
  Int_t nentries = (Int_t)T->GetEntries();
  cerr << "nentries = " << nentries << endl;

  // ----------------
  // Open output file
  // ----------------
  stringstream dirname;
  dirname << "Data" << ends;
  //dirname << "Results/10k" << ends;
  //dirname << "Results/1k" << ends;
  if (gSystem->AccessPathName(dirname.str().data())){
    gSystem->mkdir(dirname.str().data(),kTRUE);
    cerr << "Create new directory: " << dirname.str().data() << endl;
  }
  stringstream foutstr;
  foutstr << dirname.str().data() << "/run" << run << ".root" << ends;
  cerr << "Outfile: " << foutstr.str().data() << endl;
  TFile *hfp = new TFile(foutstr.str().data(),"RECREATE","Marilin");

  // --------------------
  // Create Reco. classes
  // --------------------
#if 0
  JSimpleTrackFitter::SetLayerToLookAt(layertolookat);
  if (inout == "out") {
    JSimpleTrackFitter::SetIsOutFlag();
    JTrackMaker::SetLayerToLookAt(layertolookat);
  }
#endif
  cerr << "distortion correction file = " << delxfile << endl;
  (void) JHitCorrector::GetInstancePtr(delxfile);
  JHitMaker        hitmaker;
  TKalDetCradle    lp1;
  EXTPCKalDetector tpcdet;
  lp1.Install(tpcdet);
  //lp1.SwitchOffMS();

  JSimpleTrackFitter fitter(&tpcdet);
  JTrack::SetTrackFitter(&fitter);

  JTrackMaker trackmaker;
  trackmaker.SetTrackFinder(new JTripletTrackFinder(&trackmaker));
  JTripletTrackFinder::SetMinNhits   (kMinNhits);
  JTripletTrackFinder::SetMaxMissing (kMaxMissing);
  JTripletTrackFinder::SetCosThetaCut(kCosThetaCut);
  JTripletTrackFinder::SetZdiffCut(kZdiffCut);
#if 1
#if 0
  trackmaker.KillModule(1);
  trackmaker.KillModule(6);
#else
  trackmaker.KillModule(0);
  //trackmaker.KillModule(3);
  trackmaker.KillModule(5);
#if 0
  // Exclude row 44 from the fit
  Int_t layertolookat = 16;
  JSimpleTrackFitter::SetIsOutFlag();
  JSimpleTrackFitter::SetLayerToLookAt(layertolookat);
#endif
#endif
#endif
  // --------------------
  // Now loop over events
  // --------------------
  //const Int_t kMaxEvt = 20000;
  const Int_t kMaxEvt = 2000;
  //const Int_t kMaxEvt = 1000;
  //const Int_t kMaxEvt = 100;

  std::map<std::string, TH2D *> pulsemap;

  for (Int_t ev=0; ev<nentries && ev<kMaxEvt; ev++) {
      T->GetEntry(ev);
      if (!(ev%100)) {
        cerr << "ev: " << ev << " evno: " << event->GetEventNumber() << " #cls: " << event->GetEntries() << endl;
      }
      cerr << " -------<Event " << ev << " >----------------" << endl;
      JCluster::SetSortBy(JCluster::kDet); // set sorting policy
      event->Sort(); // sort clusters in ascending order of layers
      hitmaker.MakeHits(event);
      //hitmaker.DebugPrint("detail");
      trackmaker.MakeTracks(&hitmaker);
      trackmaker.DebugPrint("detail");
      TIter next(&trackmaker);
      JTrack *tp = nullptr;
      while ((tp = static_cast<JTrack *>(next()))) {      // loop over tracks
          TIter nexthit(tp);
	  JHit *hp = nullptr;
          while ((hp = static_cast<JHit *>(nexthit()))) { // loop ver hits
              TIter nextcluster(hp);
	      double    chgmax = -1;
	      JCluster *pkcp   = nullptr;
	      JCluster *cp     = nullptr;
              while ((cp = static_cast<JCluster *>(nextcluster()))) { // loop over pulses
                  double chg = cp->GetCharge();
                  if (chg > chgmax) {
                      chgmax = chg;
		      pkcp   = cp;
		  }
              }
              double time0  = pkcp->GetStartTime();
	      double time   = pkcp->GetTime();
	      int    module = pkcp->GetModuleID();
	      int    row    = pkcp->GetLayerID();
	      int    pad    = pkcp->GetPadID();
	      std::stringstream hname;
	      hname << "hPulse" << module << "_" << row << ends; // histogram name
	      if (!pulsemap[hname.str()]) {
                  pulsemap[hname.str()] = new TH2D(hname.str().data(),"",300,-10.,20.,1200,0.,1.2);
	      }
	      auto hpp = pulsemap[hname.str()];
              for (Int_t i=0; i<pkcp->GetLength(); i++) {
                  hpp->Fill(i+time0-time, pkcp->operator[](i)/chgmax);
              }
#if 0
	      int    ipk = time - time0;
	      double fpk = pkcp->operator[](ipk)/chgmax;
              cerr << "ghost=" << (fpk>0.2 ? 1 : 0) << " module=" << module << " row=" << row << " pad=" << pad  << " ";
              cerr << "time0=" << time0 << " time=" << time << " adc=";
              for (Int_t i=0; i<pkcp->GetLength(); i++) {
                  cerr << pkcp->operator[](i) << " ";
              }
    	  cerr << endl;
#endif
	  }
      }
  } 
  // -----------------------
  // Now output tree headers
  // -----------------------
  auto hTracks = static_cast<TNtupleD *>(gROOT->FindObject("hTracks"));
  auto hResXin = static_cast<TNtupleD *>(gROOT->FindObject("hResXin"));
  auto hResZin = static_cast<TNtupleD *>(gROOT->FindObject("hResZin"));

  const Int_t kNmodules = 3;
  const Int_t kNlayers  = 28;
  TH2D *hPadRes[kNmodules*kNlayers];
  for (Int_t i=0; i<kNmodules*kNlayers; i++) {
    stringstream name;
    name << "qx" << setw(2) << setfill('0') << i << ends;
    hPadRes[i] = static_cast<TH2D *>(gROOT->FindObject(name.str().data()));
  }

  hfp->cd();
  hTracks->Write();
  hResXin->Write();
  hResZin->Write();
  for (Int_t i=0; i<kNmodules*kNlayers; i++) { 
     if (hPadRes[i]) hPadRes[i]->Write();
  }

  for (const auto& [key, hpp]: pulsemap) {
     if (hpp) hpp->Write();
  }

  hfp->Write();
}
